import java.io.IOException;
import java.io.InputStreamReader;

/**
 * Implementation of the Count program corresponding to Part 2 of the homework assignment.
 * 
 * Note: it would have been possible to implement Count via inheritance from Frequency, but the benefit
 *       would have been negligible because most of the actual program logic is in other classes anyway.
 *       Thus, it is ok to replicate the similar---but not identiccal---error checking that was already
 *       done in Frequency.
 */
public class Count {

    /**
     * The program first checks the command line arguments and opens the
     * corresponding files, and then pumps the input through the word counter. Finally, it displays the count.
     * 
     * @param args the command line arguments
     */
    public static void main(String[] args) {
        if (args.length == 1 || args.length == 2) {
            // We have the right number of arguments. Let's open the files.
            
            FilteredWordCounter wordCounter = null;
            WordIterator wordIterator = null;
            int index = 0;
            try {
                // Open word list.
                wordCounter  = new FilteredWordCounter(args[0]);
                index++;
                // Open text file.
                if (args.length == 2)
                    // Filename specified.
                    wordIterator = new WordIterator(args[1]);
                else
                    // Filename omitted; read from standard input.
                    wordIterator = new WordIterator(new InputStreamReader(System.in));
            } catch (IOException exception) {
                // Something is wrong with the file.
                System.err.printf("File '%s' could not be read: %s\n", args[index], exception.getMessage());
                // Exit with error status.
                System.exit(2);
            }
            
            // Everything is fine. Let's count.
            for (String word :  wordIterator)
                wordCounter.count(word);
            
            // We are done. Let's print the count.
            for (String word : wordCounter)
                System.out.printf("%6d %s\n", wordCounter.getCount(word), word);
        }
        else {
            // educate user
            System.err.println("Usage: java Count <wordlist> [<textfile>]");
            // Exit with error status.
            System.exit(1);
        }
    }
}
