import java.io.IOException;
import java.io.InputStreamReader;

/**
 * Implementation of the Frequency program corresponding to Part 1 of the homework assignment.
 */
public class Frequency {

    /**
     * The program first checks the command line arguments and opens the
     * corresponding files, and then pumps the input through the word counter. Finally, it displays the count.
     * 
     * @param args the command line arguments
     */
    public static void main(String[] args) {
        if (args.length == 1 || args.length == 2) {
            // We have the right number of arguments.
            
            WordCounter wordCounter = new WordCounter();
            int howMany = 0;
            
            try {
                howMany = Integer.parseInt(args[0]);
                if (howMany <= 0) {
                    System.err.printf("Not a posivitve number: %d.\n", howMany);
                    System.exit(2);
                }
            } catch (NumberFormatException exception) {
                System.err.printf("Invalid number: %s.\n", args[0]);
                System.exit(2);
            }
            
            WordIterator wordIterator = null;
            try {
                // Open text file.
                if (args.length == 2)
                    // Filename specified.
                    wordIterator = new WordIterator(args[1]);
                else
                    // Filename omitted; read from standard input.
                    wordIterator = new WordIterator(new InputStreamReader(System.in));
            } catch (IOException exception) {
                // Something is wrong with the file.
                System.err.printf("File '%s' could not be read: %s\n", args[1], exception.getMessage());
                // Exit with error status.
                System.exit(2);
            }
            
            // Everything is fine. Let's count.
            for (String word :  wordIterator)
                wordCounter.count(word);
            
            // We are done. Let's print the count for each encountered word.
            for (String word : wordCounter.getMostFrequent(howMany))
                System.out.printf("%6d %s\n", wordCounter.getCount(word), word);
        }
        else {
            // educate user
            System.err.println("Usage: java Frequency <wordlist> [<textfile>]");
            // Exit with error status.
            System.exit(1);
        }
    }
}
